################################################################################################
##### This is the R script used for the SNP power assessment simulations in Nussberger et al. 2013, Molecular Ecology Resources
##### The R script was run before each set of simulations with NewHybrids by Eric Anderson.

##### We used the no_gui version of NewHybrids (newhybridsng, which can be found on GitHub), as follows:
# ./newhybsng -d output_R.txt --no-gui
# where "output_R.txt" represents the files generated using the Rscript below (for instance "W.txt")
# for simulations with 10 categories, option "-c TwoGensGtypFreq10.txt" was added to the command.
# for each "output_R.txt" file, 200 simulations were run.

#################################################################
# number of loci, was varied between 48, 32, 24
nloc <- 48
#################################################################

setwd("/my/working/directory/")

dat <- read.table("SelectionSimulOnlyTestInd040912.txt", header=T)[,1:(nloc + 1)]
pure <- read.table("SelectionSimulOnlyRefs040912.txt")[,1:(nloc + 1)]

# index for domestic cats
HK <- 1:28

# index for wild cats
WK <- 29:37

# maximal number of alleles to be drawn (limited due to number of individuals)
max_al <- min(length(HK)*2, length(WK)*2)

# number of loci
#nloc <- ncol(dat)-1

dat[dat==0] <- "00"

########################################################################
##############        Resampling of pure W        #######################
# prepare genotype file
W <- data.frame(array(NA,dim=c(max_al/2,nloc)))

for (i in 2:ncol(dat)){
	wk <- dat[WK, i]
	wk_al <- c(substr(wk, 1, 1), substr(wk , 2, 2))
	# random permutation of alleles
	wk_al_s <- sample(wk_al, size=max_al, replace=F)
	W[,(i-1)] <- paste(wk_al_s[1:(max_al/2)], wk_al_s[((max_al/2)+1) : max_al], sep="")
}

# genotypes with one unknown allele are defined as unknown genotype
W[W=="10"] <- "0"
W[W=="20"] <- "0"
W[W=="30"] <- "0"
W[W=="40"] <- "0"
W[W=="01"] <- "0"
W[W=="02"] <- "0"
W[W=="03"] <- "0"
W[W=="04"] <- "0"
W[W=="00"] <- "0"

# write file to input folder for NewHybrids
sink("W.txt")
cat("NumIndivs ")
cat(max_al/2+nrow(pure))
cat("\n")
cat("NumLoci ")
cat(nloc)
cat("\n")
cat("Digits ")
cat(1)
cat("\n")
cat("Format Lumped ")
cat("\n")
cat("\n")
write.table(t(names(dat)), col.names=F, row.names=F, quote=F)
write.table(cbind(1:(max_al/2),rep(" ",max_al/2), W), col.names=F, row.names=F, quote=F)
write.table(cbind((max_al/2+1):(max_al/2+nrow(pure)), pure), col.names=F, row.names=F, quote=F)
sink()


########################################################################
##############        Resampling of pure D        ########################################

# prepare genotype file
D <- data.frame(array(NA,dim=c(max_al/2,nloc)))

for (i in 2:(nloc + 1)){
	hk <- dat[HK, i]
	hk_al <- c(substr(hk, 1, 1), substr(hk , 2, 2))
	# random permutation of alleles
	hk_al_s <- sample(hk_al, size=max_al, replace=F)
	D[,(i-1)] <- paste(hk_al_s[1:(max_al/2)], hk_al_s[((max_al/2)+1) : max_al], sep="")
}

# genotypes with one unknown allele are defined as unknown genotype
D[D=="10"] <- "0"
D[D=="20"] <- "0"
D[D=="30"] <- "0"
D[D=="40"] <- "0"
D[D=="01"] <- "0"
D[D=="02"] <- "0"
D[D=="03"] <- "0"
D[D=="04"] <- "0"
D[D=="00"] <- "0"

# write file to input folder for NewHybrids
sink("D.txt")
cat("NumIndivs ")
cat(max_al/2+nrow(pure))
cat("\n")
cat("NumLoci ")
cat(nloc)
cat("\n")
cat("Digits ")
cat(1)
cat("\n")
cat("Format Lumped ")
cat("\n")
cat("\n")
write.table(t(names(dat)), col.names=F, row.names=F, quote=F)
write.table(cbind(1:(max_al/2),rep(" ",max_al/2), D), col.names=F, row.names=F, quote=F)
write.table(cbind((max_al/2+1):(max_al/2+nrow(pure)), pure), col.names=F, row.names=F, quote=F)
sink()


########################################################################
##############     Simulation of F1    ########################################
# prepare genotype file of F1
F1 <- data.frame(array(NA,dim=c(max_al,nloc)))

for (i in 2:ncol(dat)){
	hk <- dat[HK, i]
	hk_al <- c(substr(hk, 1, 1), substr(hk, 2, 2))
	
	wk <- dat[WK, i]
	wk_al <- c(substr(wk, 1, 1), substr(wk , 2, 2))
	
	# random permutation of alleles and pasting into dataframe
	F1[,(i-1)] <- paste(sample(hk_al, size=max_al, replace=F), sample(wk_al, size=max_al, replace=F), sep="")
}

# genotypes with one unknown allele are defined as unknown genotype
F1[F1=="10"] <- "0"
F1[F1=="20"] <- "0"
F1[F1=="30"] <- "0"
F1[F1=="40"] <- "0"
F1[F1=="01"] <- "0"
F1[F1=="02"] <- "0"
F1[F1=="03"] <- "0"
F1[F1=="04"] <- "0"
F1[F1=="00"] <- "0"

# write file to input folder for NewHybrids
sink("F1.txt")
cat("NumIndivs ")
cat(max_al+nrow(pure))
cat("\n")
cat("NumLoci ")
cat(nloc)
cat("\n")
cat("Digits ")
cat(1)
cat("\n")
cat("Format Lumped ")
cat("\n")
cat("\n")
write.table(t(names(dat)), col.names=F, row.names=F, quote=F)
write.table(cbind(1:(max_al),rep(" ",max_al), F1), col.names=F, row.names=F, quote=F)
write.table(cbind((max_al+1):(max_al+nrow(pure)), pure), col.names=F, row.names=F, quote=F)
sink()


########################################################################
##############     Simulation of F2    ########################################

# prepare genotype file of F2
F2 <- data.frame(array(NA,dim=c(max_al,nloc)))

for (i in 2:ncol(dat)){
	hk <- dat[HK, i]
	hk_al <- c(substr(hk, 1, 1), substr(hk, 2, 2))
	# random permutation of domestic cat alleles
	hk_al_s <- sample(hk_al, replace=F)
	
	wk <- dat[WK, i]
	wk_al <- c(substr(wk, 1, 1), substr(wk, 2, 2))
	# random permutation of wildcat alleles
	wk_al_s <- sample(wk_al, replace=F)
	
	# preparation and random permutation of F1 alleles
	al <- c(hk_al_s[1:max_al], wk_al_s[1:max_al])
	al_r <- sample(al, replace=F)

	F2[,(i-1)] <- paste(al_r[1:(length(al_r)/2)], al_r[(length(al_r)/2+1):length(al_r)], sep="")	
}

# genotypes with one unknown allele are defined as unknown genotype
F2[F2=="10"] <- "0"
F2[F2=="20"] <- "0"
F2[F2=="30"] <- "0"
F2[F2=="40"] <- "0"
F2[F2=="01"] <- "0"
F2[F2=="02"] <- "0"
F2[F2=="03"] <- "0"
F2[F2=="04"] <- "0"
F2[F2=="00"] <- "0"

# write file to input folder for NewHybrids
sink("F2.txt")
cat("NumIndivs ")
cat(max_al+nrow(pure))
cat("\n")
cat("NumLoci ")
cat(nloc)
cat("\n")
cat("Digits ")
cat(1)
cat("\n")
cat("Format Lumped ")
cat("\n")
cat("\n")
write.table(t(names(dat)), col.names=F, row.names=F, quote=F)
write.table(cbind(1:(max_al),rep(" ",max_al), F2), col.names=F, row.names=F, quote=F)
write.table(cbind((max_al+1):(max_al+nrow(pure)), pure), col.names=F, row.names=F, quote=F)
sink()


########################################################################
##############      backcross into wildcat bxW   #######################

# number of samples to be sampled for F1s
f1_al <- max_al/3

# prepare genotype file of backcross F1 with WK
bxW <- data.frame(array(NA,dim=c(max_al-f1_al,nloc)))


for (i in 2:ncol(dat)){
	hk <- dat[HK, i]
	hk_al <- c(substr(hk, 1, 1), substr(hk, 2, 2))
	# random permutation of domestic cat alleles
	hk_al_s <- sample(hk_al, replace=F)
	
	wk <- dat[WK, i]
	wk_al <- c(substr(wk, 1, 1), substr(wk, 2, 2))
	# random permutation of wildcat alleles
	wk_al_s <- sample(wk_al, replace=F)
	
	# preparation and random permutation of F1 alleles
	al <- c(hk_al_s[1:f1_al], wk_al_s[1:f1_al])
	al_r <- sample(al, replace=F)
	
	# generation of bxW (backcrosses wildcats)
	bxW[,(i-1)] <- paste(al_r, wk_al_s[(f1_al+1):(max_al)], sep="")
	
}

# genotypes with one unknown allele are defined as unknown genotype
bxW[bxW=="10"] <- "0"
bxW[bxW=="20"] <- "0"
bxW[bxW=="30"] <- "0"
bxW[bxW=="40"] <- "0"
bxW[bxW=="01"] <- "0"
bxW[bxW=="02"] <- "0"
bxW[bxW=="03"] <- "0"
bxW[bxW=="04"] <- "0"
bxW[bxW=="00"] <- "0"

# write file to input folder for NewHybrids
sink("bxW.txt")
cat("NumIndivs ")
cat(max_al-f1_al+nrow(pure))
cat("\n")
cat("NumLoci ")
cat(nloc)
cat("\n")
cat("Digits ")
cat(1)
cat("\n")
cat("Format Lumped ")
cat("\n")
cat("\n")
write.table(t(names(dat)), col.names=F, row.names=F, quote=F)
write.table(cbind(1:(max_al-f1_al),rep(" ",max_al-f1_al),bxW), col.names=F, row.names=F, quote=F)
write.table(cbind(((max_al-f1_al)+1):((max_al-f1_al)+nrow(pure)), pure), col.names=F, row.names=F, quote=F)
sink()


############################################################################
##############      backcross into domestic cat bxC   #######################
# number of samples to be sampled for F1s
f1_al <- max_al/3

# prepare genotype file of backcross F1 into domestic
bxD <- data.frame(array(NA,dim=c(max_al-f1_al,nloc)))


for (i in 2:ncol(dat)){
	hk <- dat[HK, i]
	hk_al <- c(substr(hk, 1, 1), substr(hk, 2, 2))
	# random permutation of domestic cat alleles
	hk_al_s <- sample(hk_al, replace=F)
	
	wk <- dat[WK, i]
	wk_al <- c(substr(wk, 1, 1), substr(wk, 2, 2))
	# random permutation of wildcat alleles
	wk_al_s <- sample(wk_al, replace=F)

	# preparation and random permutation of F1 alleles
	al <- c(hk_al_s[1:f1_al], wk_al_s[1:f1_al])
	al_r <- sample(al, replace=F)
	
	# generation of bxD (backcrosses domestic cats)
	bxD[,(i-1)] <- paste(al_r, hk_al_s[(f1_al+1):(max_al)], sep="")	
}

# genotypes with one unknown allele are defined as unknown genotype
bxD[bxD=="10"] <- "0"
bxD[bxD=="20"] <- "0"
bxD[bxD=="30"] <- "0"
bxD[bxD=="40"] <- "0"
bxD[bxD=="01"] <- "0"
bxD[bxD=="02"] <- "0"
bxD[bxD=="03"] <- "0"
bxD[bxD=="04"] <- "0"
bxD[bxD=="00"] <- "0"

# write file to input folder for NewHybrids
sink("bxD.txt")
cat("NumIndivs ")
cat(max_al-f1_al+nrow(pure))
cat("\n")
cat("NumLoci ")
cat(nloc)
cat("\n")
cat("Digits ")
cat(1)
cat("\n")
cat("Format Lumped ")
cat("\n")
cat("\n")
write.table(t(names(dat)), col.names=F, row.names=F, quote=F)
write.table(cbind(1:(max_al-f1_al),rep(" ",(max_al-f1_al)),bxD), col.names=F, row.names=F, quote=F)
write.table(cbind(((max_al-f1_al)+1):((max_al-f1_al)+nrow(pure)), pure), col.names=F, row.names=F, quote=F)
sink()

###############################################################################
##############      back-backcross into wild cat bxWxW  #######################

# number of samples to be sampled for F1s, so that sampling without replacement is possible
f1_al <- max_al/3

# prepare genotype file
bxWxW <- data.frame(array(NA,dim=c(max_al-f1_al,nloc)))

for (i in 2:ncol(dat)){
	hk <- dat[HK, i]
	hk_al <- c(substr(hk, 1, 1), substr(hk, 2, 2))
	# random permutation of domestic cat alleles
	hk_al_s <- sample(hk_al, replace=F)
	
	wk <- dat[WK, i]
	wk_al <- c(substr(wk, 1, 1), substr(wk, 2, 2))
	# random permutation of wildcat alleles
	wk_al_s <- sample(wk_al, replace=F)

	# preparation and random permutation of F1 alleles
	f1 <- c(hk_al_s[1:f1_al], wk_al_s[1:f1_al])
	f1_s <- sample(f1, replace=F)
	
	# preparation and random permutation of bxW (backcrosses wild cats)
	bxw <- c(f1_s[1:f1_al], wk_al_s[(f1_al+1):(2*f1_al)])	
	bxw_s <- sample(bxw, replace = F)
	
	# generation of bxWxW
	bxWxW[,(i-1)] <- paste(bxw_s[1:f1_al], wk_al_s[(2*f1_al+1):(max_al)], sep="")
	
}

# genotypes with one unknown allele are defined as unknown genotype
bxWxW[bxWxW=="10"] <- "0"
bxWxW[bxWxW=="20"] <- "0"
bxWxW[bxWxW=="30"] <- "0"
bxWxW[bxWxW=="40"] <- "0"
bxWxW[bxWxW=="01"] <- "0"
bxWxW[bxWxW=="02"] <- "0"
bxWxW[bxWxW=="03"] <- "0"
bxWxW[bxWxW=="04"] <- "0"
bxWxW[bxWxW=="00"] <- "0"

# write file to input folder for NewHybrids
sink("bxWxW.txt")
cat("NumIndivs ")
cat(max_al-f1_al+nrow(pure))
cat("\n")
cat("NumLoci ")
cat(nloc)
cat("\n")
cat("Digits ")
cat(1)
cat("\n")
cat("Format Lumped ")
cat("\n")
cat("\n")
write.table(t(names(dat)), col.names=F, row.names=F, quote=F)
write.table(cbind(1:(max_al-f1_al),rep(" ",(max_al-f1_al)),bxWxW), col.names=F, row.names=F, quote=F)
write.table(cbind(((max_al-f1_al)+1):((max_al-f1_al)+nrow(pure)), pure), col.names=F, row.names=F, quote=F)
sink()


###############################################################################
##############      back-backcross into domestic cat bxDxD  #######################

# number of samples to be sampled for F1s, so that sampling without replacement is possible
f1_al <- max_al/3

# prepare genotype file
bxDxD <- data.frame(array(NA,dim=c(max_al-f1_al,nloc)))

for (i in 2:ncol(dat)){
	hk <- dat[HK, i]
	hk_al <- c(substr(hk, 1, 1), substr(hk, 2, 2))
	# random permutation of domestic cat alleles
	hk_al_s <- sample(hk_al, replace=F)
	
	wk <- dat[WK, i]
	wk_al <- c(substr(wk, 1, 1), substr(wk, 2, 2))
	# random permutation of wildcat alleles
	wk_al_s <- sample(wk_al, replace=F)

	# preparation and random permutation of F1 alleles
	f1 <- c(hk_al_s[1:f1_al], wk_al_s[1:f1_al])
	f1_s <- sample(f1, replace=F)
	
	# preparation and random permutation of bxD (backcrosses domestic cats)
	dxw <- c(f1_s[1:f1_al], hk_al_s[(f1_al+1):(2*f1_al)])	
	dxw_s <- sample(dxw, replace = F)
	
	# generation of bxDxD
	bxDxD[,(i-1)] <- paste(dxw_s[1:f1_al], hk_al_s[(2*f1_al+1):(max_al)], sep="")
	
}

# genotypes with one unknown allele are defined as unknown genotype
bxDxD[bxDxD=="10"] <- "0"
bxDxD[bxDxD=="20"] <- "0"
bxDxD[bxDxD=="30"] <- "0"
bxDxD[bxDxD=="40"] <- "0"
bxDxD[bxDxD=="01"] <- "0"
bxDxD[bxDxD=="02"] <- "0"
bxDxD[bxDxD=="03"] <- "0"
bxDxD[bxDxD=="04"] <- "0"
bxDxD[bxDxD=="00"] <- "0"

# write file to input folder for NewHybrids
sink("bxDxD.txt")
cat("NumIndivs ")
cat(max_al-f1_al+nrow(pure))
cat("\n")
cat("NumLoci ")
cat(nloc)
cat("\n")
cat("Digits ")
cat(1)
cat("\n")
cat("Format Lumped ")
cat("\n")
cat("\n")
write.table(t(names(dat)), col.names=F, row.names=F, quote=F)
write.table(cbind(1:(max_al-f1_al),rep(" ",(max_al-f1_al)),bxDxD), col.names=F, row.names=F, quote=F)
write.table(cbind(((max_al-f1_al)+1):((max_al-f1_al)+nrow(pure)), pure), col.names=F, row.names=F, quote=F)
sink()


###############################################################################
##############      back-backcross bxWxf1      #######################

# number of samples to be sampled for F1s, so that sampling without replacement is possible
f1_al <- max_al/3

# prepare genotype file
bxWxf1 <- data.frame(array(NA,dim=c(max_al-f1_al,nloc)))

for (i in 2:ncol(dat)){
	hk <- dat[HK, i]
	hk_al <- c(substr(hk, 1, 1), substr(hk, 2, 2))
	# random permutation of domestic cat alleles
	hk_al_s <- sample(hk_al, replace=F)
	
	wk <- dat[WK, i]
	wk_al <- c(substr(wk, 1, 1), substr(wk, 2, 2))
	# random permutation of wildcat alleles
	wk_al_s <- sample(wk_al, replace=F)

	# preparation and random permutation of F1 alleles
	f1 <- c(hk_al_s[1:f1_al], wk_al_s[1:f1_al])
	f1_s <- sample(f1, replace=F)
	
	# preparation and random permutation of bxW (backcrosses wild cats)
	bxw <- c(f1_s[1:f1_al], wk_al_s[(f1_al+1):(2*f1_al)])	
	bxw_s <- sample(bxw, replace = F)
	
	# generation of bxWxf1
	bxWxf1[,(i-1)] <- paste(bxw_s[1:f1_al], f1_s[(f1_al+1):(2*f1_al)], sep="")
	
}

# genotypes with one unknown allele are defined as unknown genotype
bxWxf1[bxWxf1=="10"] <- "0"
bxWxf1[bxWxf1=="20"] <- "0"
bxWxf1[bxWxf1=="30"] <- "0"
bxWxf1[bxWxf1=="40"] <- "0"
bxWxf1[bxWxf1=="01"] <- "0"
bxWxf1[bxWxf1=="02"] <- "0"
bxWxf1[bxWxf1=="03"] <- "0"
bxWxf1[bxWxf1=="04"] <- "0"
bxWxf1[bxWxf1=="00"] <- "0"

# write file to input folder for NewHybrids
sink("bxWxf1.txt")
cat("NumIndivs ")
cat(max_al-f1_al+nrow(pure))
cat("\n")
cat("NumLoci ")
cat(nloc)
cat("\n")
cat("Digits ")
cat(1)
cat("\n")
cat("Format Lumped ")
cat("\n")
cat("\n")
write.table(t(names(dat)), col.names=F, row.names=F, quote=F)
write.table(cbind(1:(max_al-f1_al),rep(" ",(max_al-f1_al)),bxWxf1), col.names=F, row.names=F, quote=F)
write.table(cbind(((max_al-f1_al)+1):((max_al-f1_al)+nrow(pure)), pure), col.names=F, row.names=F, quote=F)
sink()


###############################################################################
##############      back-backcross bxDxf1      #######################

# number of samples to be sampled for F1s, so that sampling without replacement is possible
f1_al <- max_al/3

# prepare genotype file
bxDxf1 <- data.frame(array(NA,dim=c(max_al-f1_al,nloc)))

for (i in 2:ncol(dat)){
	hk <- dat[HK, i]
	hk_al <- c(substr(hk, 1, 1), substr(hk, 2, 2))
	# random permutation of domestic cat alleles
	hk_al_s <- sample(hk_al, replace=F)
	
	wk <- dat[WK, i]
	wk_al <- c(substr(wk, 1, 1), substr(wk, 2, 2))
	# random permutation of wildcat alleles
	wk_al_s <- sample(wk_al, replace=F)

	# preparation and random permutation of F1 alleles
	f1 <- c(hk_al_s[1:f1_al], wk_al_s[1:f1_al])
	f1_s <- sample(f1, replace=F)
	
	# preparation and random permutation of bxD (backcrosses domestic cats)
	bxd <- c(f1_s[1:f1_al], hk_al_s[(f1_al+1):(2*f1_al)])	
	bxd_s <- sample(bxd, replace = F)
	
	# generation of bxDxf1
	bxDxf1[,(i-1)] <- paste(bxd_s[1:f1_al], f1_s[(f1_al+1):(2*f1_al)], sep="")	
}

# genotypes with one unknown allele are defined as unknown genotype
bxDxf1[bxDxf1=="10"] <- "0"
bxDxf1[bxDxf1=="20"] <- "0"
bxDxf1[bxDxf1=="30"] <- "0"
bxDxf1[bxDxf1=="40"] <- "0"
bxDxf1[bxDxf1=="01"] <- "0"
bxDxf1[bxDxf1=="02"] <- "0"
bxDxf1[bxDxf1=="03"] <- "0"
bxDxf1[bxDxf1=="04"] <- "0"
bxDxf1[bxDxf1=="00"] <- "0"

# write file to input folder for NewHybrids
sink("bxDxf1.txt")
cat("NumIndivs ")
cat(max_al-f1_al+nrow(pure))
cat("\n")
cat("NumLoci ")
cat(nloc)
cat("\n")
cat("Digits ")
cat(1)
cat("\n")
cat("Format Lumped ")
cat("\n")
cat("\n")
write.table(t(names(dat)), col.names=F, row.names=F, quote=F)
write.table(cbind(1:(max_al-f1_al),rep(" ",(max_al-f1_al)),bxDxf1), col.names=F, row.names=F, quote=F)
write.table(cbind(((max_al-f1_al)+1):((max_al-f1_al)+nrow(pure)), pure), col.names=F, row.names=F, quote=F)
sink()

